/**
  *  \file planetscentral/dbclient/dbint.cc
  *  \brief Database Client: Integer Values
  */

#include "planetscentral/dbclient/dbint.h"
#include "planetscentral/dbclient/dbclient.h"

DbInt::DbInt(DbClient& client, string_t s)
    : DbKey(client, s)
{ }

/** Decrement by one.
    \return new value */
int32_t
DbInt::operator--()
{
    return client.callInt(DbRequest().withString("DECR").withString(key));
}

/** Decrement by %val.
    \return new value */
int32_t
DbInt::operator-=(int32_t val)
{
    return client.callInt(DbRequest().withString("DECRBY").withString(key).withInt(val));
}

/** Get value. */
int32_t
DbInt::get()
{
    return client.callInt(DbRequest().withString("GET").withString(key));
}

/** Replace by a new value.
    \return old value */
int32_t
DbInt::replaceBy(int32_t newValue)
{
    return client.callInt(DbRequest().withString("GETSET").withString(key).withInt(newValue));
}

/** Increment by one.
    \return new value */
int32_t
DbInt::operator++()
{
    return client.callInt(DbRequest().withString("INCR").withString(key));
}

/** Increment by %val.
    \return new value */
int32_t
DbInt::operator+=(int32_t val)
{
    return client.callInt(DbRequest().withString("INCRBY").withString(key).withInt(val));
}

/** Set value
    \param newValue new value */
void
DbInt::set(int32_t newValue)
{
    client.callVoid(DbRequest().withString("SET").withString(key).withInt(newValue));
}

/** Create anew.
    \param newValue value to use
    \retval true a new value has been created
    \retval false this value already exists */
bool
DbInt::setNew(int32_t newValue)
{
    return client.callInt(DbRequest().withString("SETNX").withString(key).withInt(newValue));
}
