/**
  *  \file planetscentral/dbclient/dbsubtree.cc
  *  \brief Database Client: Subtrees
  */

#include "planetscentral/dbclient/dbsubtree.h"
#include "cpluslib/format.h"

/** Constructor.
    \param client Database client
    \param prefix Prefix for this subtree, e.g. "foo:bar:" */
DbSubtree::DbSubtree(DbClient& client, string_t prefix)
    : client(client), prefix(prefix)
{ }

/** Constructor.
    \param client Database client
    \param prefix Prefix for this subtree, e.g. "foo:bar:" */
DbSubtree::DbSubtree(DbClient& client, const char* prefix)
    : client(client), prefix(prefix)
{ }

DbSubtree::~DbSubtree()
{ }

/** Access to a hash key.
    \param name Name of key */
DbHash
DbSubtree::hashKey(const string_t& name) const
{
    return DbHash(client, prefix + name);
}

/** Access to a hash key.
    \param name Name of key */
DbHash
DbSubtree::hashKey(const char* name) const
{
    return DbHash(client, prefix + name);
}

/** Access to an integer key.
    \param name Name of key */
DbInt
DbSubtree::intKey(const string_t& name) const
{
    return DbInt(client, prefix + name);
}

/** Access to an integer key.
    \param name Name of key */
DbInt
DbSubtree::intKey(const char* name) const
{
    return DbInt(client, prefix + name);
}

/** Access to an integer set key.
    \param name Name of key */
DbIntSet
DbSubtree::intSetKey(const string_t& name) const
{
    return DbIntSet(client, prefix + name);
}

/** Access to an integer set key.
    \param name Name of key */
DbIntSet
DbSubtree::intSetKey(const char* name) const
{
    return DbIntSet(client, prefix + name);
}

/** Access to a string set key.
    \param name Name of key */
DbStringSet
DbSubtree::stringSetKey(const string_t& name) const
{
    return DbStringSet(client, prefix + name);
}

/** Access to a string set key.
    \param name Name of key */
DbStringSet
DbSubtree::stringSetKey(const char* name) const
{
    return DbStringSet(client, prefix + name);
}

/** Access to a zset key.
    \param name Name of key */
DbZSet
DbSubtree::zsetKey(const string_t& name) const
{
    return DbZSet(client, prefix + name);
}

/** Access to a zset key.
    \param name Name of key */
DbZSet
DbSubtree::zsetKey(const char* name) const
{
    return DbZSet(client, prefix + name);
}

/** Access to a string key.
    \param name Name of key */
DbString
DbSubtree::stringKey(const string_t& name) const
{
    return DbString(client, prefix + name);
}

/** Access to a string key.
    \param name Name of key */
DbString
DbSubtree::stringKey(const char* name) const
{
    return DbString(client, prefix + name);
}

/** Access to an integer list key.
    \param name Name of key */
DbIntList
DbSubtree::intListKey(const string_t& name) const
{
    return DbIntList(client, prefix + name);
}

/** Access to an integer list key.
    \param name Name of key */
DbIntList
DbSubtree::intListKey(const char* name) const
{
    return DbIntList(client, prefix + name);
}

/** Access to a string list key.
    \param name Name of key */
DbStringList
DbSubtree::stringListKey(const string_t& name) const
{
    return DbStringList(client, prefix + name);
}

/** Access to a string list key.
    \param name Name of key */
DbStringList
DbSubtree::stringListKey(const char* name) const
{
    return DbStringList(client, prefix + name);
}

/** Access to a subtree.
    \param name Name of key */
DbSubtree
DbSubtree::subtree(const string_t& name) const
{
    return DbSubtree(client, prefix + name + ":");
}

/** Access to a subtree.
    \param name Name of key */
DbSubtree
DbSubtree::subtree(const char* name) const
{
    return DbSubtree(client, prefix + name + ":");
}

/** Access to a subtree.
    \param name Name of key (will be stringified) */
DbSubtree
DbSubtree::subtree(int32_t name) const
{
    return DbSubtree(client, prefix + format("%d", name) + ":");
}
