/**
  *  \file planetscentral/host/game.h
  *  \brief Host: Game operations
  */
#ifndef C2SERVER_PLANETSCENTRAL_HOST_GAME_H
#define C2SERVER_PLANETSCENTRAL_HOST_GAME_H

#include "planetscentral/dbclient/dbsubtree.h"
#include "game/racename.h"
#include "game/playerset.h"

/** Handle to a game. Provides operations on games in the database. */
class Game {
 public:
    class Slot {
     public:
        Slot(DbSubtree tree);
        DbStringList players();
        DbHashIntField slotStatus();
        DbHashIntField turnStatus();
        DbHashIntField rank();
     private:
        DbSubtree tree;
    };

    enum PermissionLevel {
        ReadPermission,
        ConfigPermission,
        AdminPermission
    };
    enum NoExistanceCheck_t {
        NoExistanceCheck
    };

    Game(DbSubtree root, int32_t gid);
    Game(DbSubtree root, int32_t gid, NoExistanceCheck_t t);

    int32_t  getId() const;
    string_t getState();
    void     setState(string_t newState);
    string_t getType();
    void     setType(string_t newType);
    string_t getName();
    void     setName(string_t newName);
    string_t getOwner();
    void     setOwner(string_t newOwner, DbSubtree userRoot);
    string_t getConfig(string_t name);
    void     setConfig(string_t name, string_t value);
    IntValue* getConfig(const StringList_t& names);
    void     setConfig(const StringList_t& kv);
    int32_t  getConfigInt(string_t name);
    void     setConfigInt(string_t name, int32_t value);
    void     removeConfig(string_t name);
    string_t getDirectory();
    void     addGameHistoryItem(string_t what, string_t args, bool global);
    void     addUserHistoryItem(string_t what, string_t args, DbSubtree userRoot, string_t player);
    string_t getPlayerConfig(string_t player, string_t name);
    void     setPlayerConfig(string_t player, string_t name, string_t value);
    int32_t  getPlayerConfigInt(string_t player, string_t name);
    void     setPlayerConfigInt(string_t player, string_t name, int32_t value);
    string_t getRefereeScoreName();

    Slot     getSlot(int32_t slot);
    bool     isSlotInGame(int32_t slot);
    bool     isSlotPlayed(int32_t slot);
    bool     hasAnyOpenSlot();
    void     pushPlayerSlot(int32_t slot, string_t player, DbSubtree userRoot);
    string_t popPlayerSlot(int32_t slot, DbSubtree userRoot);
    void     listPlayers(int32_t slot, StringList_t& players);
    GPlayerSet getSlotsByPlayer(string_t player);
    GPlayerSet getGameSlots();

    void     clearCache();
    int32_t  getDifficulty();

    void     markBroken(string_t message);

    DbSubtree getSchedule();
    DbSubtree getGame();

    bool     isUserOnGame(string_t user);
    bool     isUserOnGameAsPrimary(string_t user);
    bool     isRankingDisabled();
    bool     isMultiJoinAllowed();

    IntValue* describe(bool verbose, string_t forUser);
    IntValue* describeSlot(int32_t slot, string_t forUser);

    bool     hasPermission(string_t user, PermissionLevel level);

    void     loadRaceNames(DbClient& dbConnection);

 private:
    DbSubtree root;
    DbSubtree game;
    int32_t   gameId;
    GRaceNameList raceNames;
};

#endif
